package de.jottyfan.timetrack.modules.done;

import static de.jottyfan.timetrack.db.done.Tables.T_DONE;
import static de.jottyfan.timetrack.db.done.Tables.T_JOB;
import static de.jottyfan.timetrack.db.done.Tables.T_MODULE;
import static de.jottyfan.timetrack.db.done.Tables.T_PROJECT;
import static de.jottyfan.timetrack.db.done.Tables.V_TASKLIST;
import static de.jottyfan.timetrack.db.done.Tables.V_TOTALOFDAY;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.jooq.DeleteConditionStep;
import org.jooq.InsertValuesStep6;
import org.jooq.Record;
import org.jooq.Record4;
import org.jooq.Record5;
import org.jooq.SelectConditionStep;
import org.jooq.SelectWhereStep;
import org.jooq.UpdateConditionStep;
import org.jooq.exception.DataAccessException;

import de.jooqFaces.JooqFacesContext;
import de.jottyfan.timetrack.db.done.tables.records.TDoneRecord;
import de.jottyfan.timetrack.db.done.tables.records.TJobRecord;
import de.jottyfan.timetrack.db.done.tables.records.TModuleRecord;
import de.jottyfan.timetrack.db.done.tables.records.TProjectRecord;
import de.jottyfan.timetrack.modules.JooqGateway;
import net.bootsfaces.component.fullCalendar.FullCalendarEventBean;
import net.bootsfaces.component.fullCalendar.FullCalendarEventList;

/**
 * 
 * @author henkej
 *
 */
public class DoneGateway extends JooqGateway {
	private final static Logger LOGGER = LogManager.getLogger(DoneGateway.class);
	private final Integer requested;

	public DoneGateway(JooqFacesContext facesContext) {
		super(facesContext);
		String dailyMinutes = facesContext.getExternalContext().getInitParameter("requestedDailyMinutes");
		this.requested = dailyMinutes == null ? 480 : Integer.valueOf(dailyMinutes); // defaults to 8 hours/day 
	}

	/**
	 * get all modules from db
	 * 
	 * @return modules
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 */
	public List<TModuleRecord> getAllModules() throws DataAccessException, ClassNotFoundException, SQLException {
		try (DSLContext jooq = getJooq()) {
			List<TModuleRecord> list = new ArrayList<>();
			SelectWhereStep<TModuleRecord> sql = jooq.selectFrom(T_MODULE);
			LOGGER.debug(sql.toString());
			for (TModuleRecord r : sql.fetch()) {
				list.add(r);
			}
			list.sort((o1, o2) -> o1 == null || o2 == null || o1.getName() == null || o2.getName() == null ? 0
					: o1.getName().compareTo(o2.getName()));
			return list;
		}
	}

	/**
	 * get all activities from db
	 * 
	 * @return activities
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 */
	public List<TJobRecord> getAllActivities() throws DataAccessException, ClassNotFoundException, SQLException {
		try (DSLContext jooq = getJooq()) {
			List<TJobRecord> list = new ArrayList<>();
			SelectWhereStep<TJobRecord> sql = jooq.selectFrom(T_JOB);
			LOGGER.debug(sql.toString());
			for (TJobRecord r : sql.fetch()) {
				list.add(r);
			}
			list.sort((o1, o2) -> o1 == null || o2 == null || o1.getName() == null || o2.getName() == null ? 0
					: o1.getName().compareTo(o2.getName()));
			return list;
		}
	}

	/**
	 * get all projects from db
	 * 
	 * @return projects
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 */
	public List<TProjectRecord> getAllProjects() throws DataAccessException, ClassNotFoundException, SQLException {
		try (DSLContext jooq = getJooq()) {
			List<TProjectRecord> list = new ArrayList<>();
			SelectWhereStep<TProjectRecord> sql = jooq.selectFrom(T_PROJECT);
			LOGGER.debug(sql.toString());
			for (TProjectRecord r : sql.fetch()) {
				list.add(r);
			}
			list.sort((o1, o2) -> o1 == null || o2 == null || o1.getName() == null || o2.getName() == null ? 0
					: o1.getName().compareTo(o2.getName()));
			return list;
		}
	}

	private Map<Integer, TProjectRecord> generateProjectMap(List<TProjectRecord> list) {
		Map<Integer, TProjectRecord> map = new HashMap<>();
		for (TProjectRecord r : list) {
			map.put(r.getPk(), r);
		}
		return map;
	}

	private Map<Integer, TModuleRecord> generateModuleMap(List<TModuleRecord> list) {
		Map<Integer, TModuleRecord> map = new HashMap<>();
		for (TModuleRecord r : list) {
			map.put(r.getPk(), r);
		}
		return map;
	}

	private Map<Integer, TJobRecord> generateJobMap(List<TJobRecord> list) {
		Map<Integer, TJobRecord> map = new HashMap<>();
		for (TJobRecord r : list) {
			map.put(r.getPk(), r);
		}
		return map;
	}

	/**
	 * get all from t_done of the given day
	 * 
	 * @param day
	 *          the day; if null, the current date is used
	 * 
	 * @return a list of found times, an empty one at least
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 */
	public List<DoneBean> getAll(LocalDateTime day) throws DataAccessException, ClassNotFoundException, SQLException {
		Map<Integer, TProjectRecord> projectMap = generateProjectMap(getAllProjects());
		Map<Integer, TModuleRecord> moduleMap = generateModuleMap(getAllModules());
		Map<Integer, TJobRecord> jobMap = generateJobMap(getAllActivities());

		if (day == null) {
			day = LocalDateTime.now();
		}
		LocalDateTime yesterday = day.minusDays(1).withHour(23).withMinute(0).withSecond(0).withNano(0);
		LocalDateTime tomorrow = day.plusDays(1).withHour(0).withMinute(0).withSecond(0).withNano(0);

		List<DoneBean> list = new ArrayList<>();
		try (DSLContext jooq = getJooq()) {
			SelectConditionStep<TDoneRecord> sql = getJooq()
			// @formatter:off
				.selectFrom(T_DONE)
				.where(T_DONE.FK_LOGIN.eq(getFkLogin()))
				.and(T_DONE.TIME_FROM.isNull()
					.or(T_DONE.TIME_FROM.greaterThan(Timestamp.valueOf(yesterday))
						.and(T_DONE.TIME_FROM.lessThan(Timestamp.valueOf(tomorrow)))))
				.and(T_DONE.TIME_UNTIL.isNull()
					.or(T_DONE.TIME_UNTIL.lessThan(Timestamp.valueOf(tomorrow))
						.and(T_DONE.TIME_UNTIL.greaterThan(Timestamp.valueOf(yesterday)))));
			// @formatter:on
			LOGGER.debug(sql.toString());
			for (TDoneRecord r : sql.fetch()) {
				list.add(new DoneBean(r, projectMap, moduleMap, jobMap));
			}
		}
		list.sort((o1, o2) -> o1 == null || o2 == null ? 0 : o1.compareTo(o2));
		return list;
	}

	/**
	 * insert data into t_done
	 * 
	 * @param bean
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 */
	public void insert(DoneBean bean) throws DataAccessException, ClassNotFoundException, SQLException {
		Integer fkProject = bean.getProject() == null ? null : bean.getProject().getPk();
		Integer fkModule = bean.getModule() == null ? null : bean.getModule().getPk();
		Integer fkJob = bean.getActivity() == null ? null : bean.getActivity().getPk();
		Integer fkLogin = getFkLogin();

		try (DSLContext jooq = getJooq()) {
			InsertValuesStep6<TDoneRecord, Timestamp, Timestamp, Integer, Integer, Integer, Integer> sql = jooq
			// @formatter:off
				.insertInto(T_DONE, 
										T_DONE.TIME_FROM, 
										T_DONE.TIME_UNTIL, 
										T_DONE.FK_PROJECT, 
										T_DONE.FK_MODULE, 
										T_DONE.FK_JOB,
										T_DONE.FK_LOGIN)
				.values(bean.getTimeFrom(), bean.getTimeUntil(), fkProject, fkModule, fkJob, fkLogin);
			// @formatter:on
			LOGGER.debug(sql.toString());
			sql.execute();
		}
	}

	/**
	 * update bean in t_done
	 * 
	 * @param bean
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 */
	public void update(DoneBean bean) throws DataAccessException, ClassNotFoundException, SQLException {
		try (DSLContext jooq = getJooq()) {
			UpdateConditionStep<TDoneRecord> sql = jooq
			// @formatter:off
				.update(T_DONE)
				.set(T_DONE.TIME_FROM, bean.getTimeFrom())
				.set(T_DONE.TIME_UNTIL, bean.getTimeUntil())
				.set(T_DONE.FK_PROJECT, bean.getProject() == null ? null : bean.getProject().getPk())
				.set(T_DONE.FK_JOB, bean.getActivity() == null ? null : bean.getActivity().getPk())
				.set(T_DONE.FK_MODULE, bean.getModule() == null ? null : bean.getModule().getPk())
				.where(T_DONE.PK.eq(bean.getPk()));
			// @formatter:on
			LOGGER.debug(sql.toString());
			sql.execute();
		}
	}

	/**
	 * delete bean from t_done
	 * 
	 * @param bean
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 */
	public void delete(DoneBean bean) throws DataAccessException, ClassNotFoundException, SQLException {
		try (DSLContext jooq = getJooq()) {
			DeleteConditionStep<TDoneRecord> sql = jooq.deleteFrom(T_DONE).where(T_DONE.PK.eq(bean.getPk()));
			LOGGER.debug(sql.toString());
			sql.execute();
		}
	}

	/**
	 * get day summary
	 * 
	 * @param day
	 *          the day
	 * @return the day summary if found, an empty map otherwise
	 * @throws SQLException 
	 * @throws ClassNotFoundException 
	 * @throws DataAccessException 
	 */
	public WholeDaySummaryBean getDaySummary(Date day) throws DataAccessException, ClassNotFoundException, SQLException {
		try (DSLContext jooq = getJooq()) {
			SelectConditionStep<Record4<String, String, String, String>> sql = jooq
			// @formatter:off
				.select(V_TOTALOFDAY.STARTTIME,
								V_TOTALOFDAY.ENDTIME,
						    V_TOTALOFDAY.WORKTIME,
						    V_TOTALOFDAY.BREAKTIME)
				.from(V_TOTALOFDAY)
				.where(V_TOTALOFDAY.DAY.eq(new SimpleDateFormat("yyyy-MM-dd").format(day)))
				.and(V_TOTALOFDAY.FK_LOGIN.eq(getFkLogin()));
			// @formatter:on
			LOGGER.debug(sql.toString());
			Record r = sql.fetchOne();
			if (r != null) {
				String startTime = r.get(V_TOTALOFDAY.STARTTIME);
				String endTime = r.get(V_TOTALOFDAY.ENDTIME);
				String workTime = r.get(V_TOTALOFDAY.WORKTIME);
				String breakTime = r.get(V_TOTALOFDAY.BREAKTIME);
				return new WholeDaySummaryBean(startTime, endTime, workTime, breakTime, requested);
			}
			return new WholeDaySummaryBean("", "", "", "", requested);
		}
	}

	/**
	 * get all jobs of day
	 * 
	 * @param day
	 *          the day
	 * @return list of found jobs; an empty list at least
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 * @throws DataAccessException
	 */
	public List<DailySummaryBean> getAllJobs(Date day) throws DataAccessException, ClassNotFoundException, SQLException {
		try (DSLContext jooq = getJooq()) {
			SelectConditionStep<Record4<String, String, String, String>> sql = jooq
			// @formatter:off
				.select(V_TASKLIST.DURATION,
								V_TASKLIST.PROJECT_NAME,
								V_TASKLIST.MODULE_NAME,
								V_TASKLIST.JOB_NAME)
				.from(V_TASKLIST)
				.where(V_TASKLIST.DAY.eq(new SimpleDateFormat("yyyy-MM-dd").format(day)))
				.and(V_TASKLIST.FK_LOGIN.eq(getFkLogin()));
			// @formatter:on
			LOGGER.debug(sql.toString());
			List<DailySummaryBean> list = new ArrayList<>();
			for (Record4<String, String, String, String> r : sql.fetch()) {
				String duration = r.get(V_TASKLIST.DURATION);
				String projectName = r.get(V_TASKLIST.PROJECT_NAME);
				String moduleName = r.get(V_TASKLIST.MODULE_NAME);
				String jobName = r.get(V_TASKLIST.JOB_NAME);
				list.add(new DailySummaryBean(projectName, moduleName, jobName, duration));
			}
			return list;
		}
	}

	/**
	 * get json representation of all calendar events of user
	 * 
	 * @return
	 * @throws SQLException 
	 * @throws ClassNotFoundException 
	 * @throws DataAccessException 
	 */
	public String getAllCalendarEvents() throws DataAccessException, ClassNotFoundException, SQLException {
		try (DSLContext jooq = getJooq()) {
			SelectConditionStep<Record5<Timestamp, Timestamp, String, String, String>> sql = jooq
			// @formatter:off
				.select(T_DONE.TIME_FROM,
								T_DONE.TIME_UNTIL,
								T_PROJECT.NAME,
								T_MODULE.NAME,
								T_JOB.NAME)
				.from(T_DONE)
				.leftJoin(T_PROJECT).on(T_PROJECT.PK.eq(T_DONE.FK_PROJECT))
				.leftJoin(T_MODULE).on(T_MODULE.PK.eq(T_DONE.FK_MODULE))
				.leftJoin(T_JOB).on(T_JOB.PK.eq(T_DONE.FK_JOB))
				.where(T_DONE.FK_LOGIN.eq(getFkLogin()));
			// @formatter:on
			LOGGER.debug(sql.toString());
			FullCalendarEventList list = new FullCalendarEventList();
			for (Record r : sql.fetch()) {
				String projectName = r.get(T_PROJECT.NAME);
				String moduleName = r.get(T_MODULE.NAME);
				String jobName = r.get(T_JOB.NAME);
				Date timeFrom = r.get(T_DONE.TIME_FROM);
				Date timeUntil = r.get(T_DONE.TIME_UNTIL);

				StringBuilder buf = new StringBuilder();
				buf.append(projectName);
				buf.append(", ");
				buf.append(moduleName);
				buf.append(": ");
				buf.append(jobName);

				FullCalendarEventBean bean = new FullCalendarEventBean(buf.toString(), timeFrom) {
					private static final long serialVersionUID = 1L;

					@Override
					public void addExtendedFields(StringBuilder buf) {
					}
				};
				bean.setEnd(timeUntil);
				bean.setColor(new RgbColor().determineRgbColor(projectName, moduleName, jobName));
				list.getList().add(bean);
			}
			return list.toJson();
		}
	}
}
