package de.jottyfan.timetrack.modules.done;

import java.io.Serializable;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.TimeZone;

import javax.enterprise.context.SessionScoped;
import javax.faces.application.FacesMessage;
import javax.inject.Named;

import org.jooq.exception.DataAccessException;

import de.jooqFaces.JooqFacesContext;
import de.jottyfan.timetrack.db.done.tables.records.TJobRecord;
import de.jottyfan.timetrack.db.done.tables.records.TModuleRecord;
import de.jottyfan.timetrack.db.done.tables.records.TProjectRecord;
import de.jottyfan.timetrack.modules.Model;

/**
 * 
 * @author henkej
 *
 */
@Named
@SessionScoped
public class DoneModel implements Model, Serializable {
	private static final long serialVersionUID = 1L;

	private DoneBean bean;
	private List<DoneBean> beans;
	private List<TProjectRecord> projects;
	private List<TModuleRecord> modules;
	private List<TJobRecord> activities;
	private List<TimeBean> times;
	private List<DailySummaryBean> allJobs;
	private WholeDaySummaryBean daySummary;
	private Date day;
	private String calendarEvents;

	public boolean init(JooqFacesContext facesContext) {
		try {
			day = day == null ? new Date() : day;
			beans = getAllOfDay(facesContext, day);
			DoneGateway gw = new DoneGateway(facesContext);
			modules = gw.getAllModules();
			activities = gw.getAllActivities();
			projects = gw.getAllProjects();
			daySummary = gw.getDaySummary(day);
			allJobs = gw.getAllJobs(day);
			calendarEvents = gw.getAllCalendarEvents();
			return true;
		} catch (DataAccessException | ClassNotFoundException | SQLException e) {
			facesContext.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "error", e.getMessage()));
			return false;
		}
	}

	public boolean loadDefaults(JooqFacesContext facesContext) {
		try {
			defineTimes();
			return true;
		} catch (DataAccessException e) {
			facesContext.addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "error loading defaults", e.getMessage()));
			return false;
		}
	}

	private void defineTimes() {
		times = new ArrayList<>();
		Integer currentHour = getCurrentTime().get(GregorianCalendar.HOUR_OF_DAY);
		Integer currentMinute = getCurrentTime().get(GregorianCalendar.MINUTE);
		if (currentMinute < 15) {
			currentMinute = 0;
		} else if (currentMinute < 30) {
			currentMinute = 15;
		} else if (currentMinute < 45) {
			currentMinute = 30;
		} else {
			currentMinute = 45;
		}
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute - 120), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute - 105), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute - 90), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute - 75), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute - 60), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute - 45), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute - 30), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute - 15), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute), "default"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute + 15), "default"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute + 30), "link"));
		times.add(new TimeBean(defineNewTime(currentHour, currentMinute + 45), "link"));
	}

	private Calendar getCurrentTime() {
		return new GregorianCalendar(TimeZone.getTimeZone("Europe/Berlin"));
	}

	private Calendar defineNewTime(Integer hours, Integer minutes) {
		Calendar cal = new GregorianCalendar();
		if (minutes < 0) {
			hours--;
			minutes += 60;
		} else if (minutes > 59) {
			hours++;
			minutes -= 60;
		}
		cal.set(0, 0, 0, hours, minutes, 0);
		return cal;
	}

	/**
	 * get all entries of t_done
	 * 
	 * @param facesContext
	 * @param day
	 *          the day to look up for
	 * @param login
	 *          the user to look up for
	 * @return all entries
	 * @throws SQLException 
	 * @throws ClassNotFoundException 
	 */
	private List<DoneBean> getAllOfDay(JooqFacesContext facesContext, Date day) throws DataAccessException, ClassNotFoundException, SQLException {
		LocalDateTime ldt = day.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
		return new DoneGateway(facesContext).getAll(ldt);
	}

	public boolean insert(JooqFacesContext facesContext) {
		try {
			bean.setDay(day);
			new DoneGateway(facesContext).insert(bean);
			return true;
		} catch (DataAccessException | ClassNotFoundException | SQLException e) {
			facesContext.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "error", e.getMessage()));
			return false;
		}
	}

	public boolean update(JooqFacesContext facesContext) {
		try {
			new DoneGateway(facesContext).update(bean);
			return true;
		} catch (DataAccessException | ClassNotFoundException | SQLException e) {
			facesContext.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "error", e.getMessage()));
			return false;
		}
	}

	public boolean delete(JooqFacesContext facesContext) {
		try {
			new DoneGateway(facesContext).delete(bean);
			return true;
		} catch (DataAccessException | ClassNotFoundException | SQLException e) {
			facesContext.addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "error", e.getMessage()));
			return false;
		}
	}

	public String getAttach() {
		StringBuilder buf = new StringBuilder();
		String thatday = new SimpleDateFormat("dd.MM.yyyy").format(day);
		for (DailySummaryBean sdb : allJobs) {
			buf.append(thatday).append("\t");
			buf.append(sdb.getDuration()).append("\t");
			buf.append(sdb.getProjectName()).append("\t");
			buf.append(sdb.getModuleName()).append("\t");
			buf.append(sdb.getJobName()).append("\t");
			buf.append("\n");
		}		
		return buf.toString();
	}
	
	public String getDayIso8601() {
		return day == null ? "" : new SimpleDateFormat("yyyy-MM-dd").format(day);
	}
	
	public void setBean(DoneBean bean) {
		this.bean = bean;
	}

	public DoneBean getBean() {
		return bean;
	}

	public List<TProjectRecord> getProjects() {
		return projects;
	}

	public List<TModuleRecord> getModules() {
		return modules;
	}

	public List<TJobRecord> getActivities() {
		return activities;
	}

	public List<DailySummaryBean> getAllJobs() {
		return allJobs;
	}

	public WholeDaySummaryBean getDaySummary() {
		return daySummary;
	}

	public List<DoneBean> getBeans() {
		return beans;
	}

	public List<TimeBean> getTimes() {
		return times;
	}

	public Date getDay() {
		return day;
	}

	public void setDay(Date day) {
		this.day = day;
	}

	public String getCalendarEvents() {
		return calendarEvents;
	}
}
